/*
   Copyright (c) 2010 - 2013 Javaika.com by Alexander Wait.
   
   All Rights Reserved.
*/



package main;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import required.ColorFunctions;
import required.DigitalMetronome;
import required.Domain;
import required.Message;


//********************************************* START CLASS *********************************************//


/**
 * This class builds the digital metronome GUI.
 * 
 * @author Alexander Wait
 * @version 2012-05-05
 */
public class DigitalMetronomeGui extends JApplet
{
   private static final String copyright = "Copyright (c) 2012 by Alexander Wait - All Rights Reserved";

   private static final long serialVersionUID = 1L;
   
   
   public static boolean isApplication = false;
   
   public static boolean isVisible = false;
   
   
   private static final int FRAME_SIZE_X = 420;
   
   private static final int FRAME_SIZE_Y = 555;


   //************************************** START INITIALISATION CODE **************************************//

   
   /**
    * Browser constructor.
    * ESCA-JAVA0166:
    */
   public void init()
   {
      if (!isApplication) 
      { 
         String copyrightParam = getParameter("copyright");
         
         if ((copyrightParam == null) || !copyrightParam.equals(copyright)) 
         {
            Message.showMessage("Invalid Copyright", "WARNING", "warning"); 
         }       
         
         else if (!getDocumentBase().getHost().equals(Domain.getDomain())) 
         {
            Message.showMessage("Unauthorised Applet Use", "ERROR", "error");
         }
        
         else
         {
            initialiser(); includeResizeEvent(); 
            
            setSize(FRAME_SIZE_X, FRAME_SIZE_Y);
         }
      }
   }
 
   
   
   /**
    * Application constructor.
    */
   public DigitalMetronomeGui()
   {
      if (isApplication) 
      {
         initialiser(); 
         
         
         frame = new JFrame(); 
         
         
         frame.setMinimumSize(new Dimension(FRAME_SIZE_X, FRAME_SIZE_Y));
        
         frame.setSize(FRAME_SIZE_X, FRAME_SIZE_Y); 
         
         frame.setLocationRelativeTo(null); 
         
         frame.setTitle("Metronome Program"); 
         
         frame.add(contentPane);
         
         
         setVisibility(); 
      }
 
   }
   
   
   
   /**
    * GUI initializer.
    * ESCA-JAVA0076:
    */
   private void initialiser()
   {
      metronome = new DigitalMetronome(); 
      
      
      
      // Initialize content pane and layout.
      
      contentPane = getContentPane(); 
      
      contentPane.setLayout(new GridBagLayout());
      
      
      
      // Initialize text field GUI components.
      
      beatDisplay = new JTextField(); beatDisplay.setEnabled(false);
      
      beatDisplay.setFont(new Font("Times New Roman", Font.PLAIN, 105));
      
      beatDisplay.setBorder(BorderFactory.createEtchedBorder());
      
      beatDisplay.setHorizontalAlignment(JTextField.CENTER);
      
      beatDisplay.setText("0");
      
      
      cycleDisplay = new JTextField(); cycleDisplay.setEnabled(false);
      
      cycleDisplay.setFont(new Font("Times New Roman", Font.PLAIN, 105));
      
      cycleDisplay.setBorder(BorderFactory.createEtchedBorder());
      
      cycleDisplay.setHorizontalAlignment(JTextField.CENTER);
      
      cycleDisplay.setText("0");
      
      
      speedDisplay = new JTextField(); speedDisplay.setEnabled(false);
      
      speedDisplay.setFont(new Font("Dialog", Font.BOLD, 12));

      speedDisplay.setBorder(BorderFactory.createEtchedBorder());
      
      speedDisplay.setHorizontalAlignment(JTextField.CENTER);
      
      speedDisplay.setText("Andante");
      
      
      beatsMinuteDisplay = new JTextField(); beatsMinuteDisplay.setEnabled(false); 
      
      beatsMinuteDisplay.setFont(new Font("Dialog", Font.BOLD, 12));
      
      beatsMinuteDisplay.setBorder(BorderFactory.createEtchedBorder());
      
      beatsMinuteDisplay.setHorizontalAlignment(JTextField.CENTER);
      
      beatsMinuteDisplay.setText("95");
      
      
      beatsCycleDisplay = new JTextField(); beatsCycleDisplay.setEnabled(false);
      
      beatsCycleDisplay.setFont(new Font("Dialog", Font.BOLD, 12));

      beatsCycleDisplay.setBorder(BorderFactory.createEtchedBorder());
      
      beatsCycleDisplay.setHorizontalAlignment(JTextField.CENTER);
      
      beatsCycleDisplay.setText("50");
      
      
      
      // Initialize JPanel GUI components.
  
      buttonPanel = new JPanel(); 
      
      buttonPanel.setLayout (new FlowLayout(FlowLayout.LEADING));
      
      buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
      
      
      
      // Initialize slider GUI components.
      
      beatsMinuteSlider = new JSlider();
      
      beatsMinuteSlider.setMajorTickSpacing(5); beatsMinuteSlider.setPaintTicks(true);
      
      beatsMinuteSlider.setBorder(BorderFactory.createTitledBorder("Beats Per Minute"));
      
      beatsMinuteSlider.addChangeListener(new BeatsMinuteChangeListener());
      
      
      beatsCycleSlider = new JSlider();

      beatsCycleSlider.setMajorTickSpacing(5); beatsCycleSlider.setPaintTicks(true);
      
      beatsCycleSlider.setBorder(BorderFactory.createTitledBorder("Beats Per Cycle"));
      
      beatsCycleSlider.addChangeListener(new BeatsCycleChangeListener());
      
      
      
      // Initialize button GUI components.
      
      start = new JButton(new ImageIcon(this.getClass().getResource
      (
         "/files/images/MetronomeStartNormal.png"))
      );
      
      
      start.setRolloverIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/MetronomeStartRollover.png"))
      );

      
      start.setPressedIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/MetronomeStartPressed.png"))
      );
      
      
      start.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));

      start.addActionListener(new StartButtonListener());
      

      stop = new JButton(new ImageIcon(this.getClass().getResource
      (
         "/files/images/MetronomeStopNormal.png"))
      );


      stop.setRolloverIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/MetronomeStopRollover.png"))
      );


      stop.setPressedIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/MetronomeStopPressed.png"))
      );


      stop.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));

      stop.addActionListener(new StopButtonListener());
      
      
      
      // Add GUI components to JPanels.
      
      buttonPanel.add(start); buttonPanel.add(stop);
      
      

      // Method calls for GUI initialization.

      gridBagConstraints(); setColors();
   }
   
   
   
   /**
    * Sets the background color.
    */
   private void setColors()
   {
      contentPane.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      buttonPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      beatDisplay.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      cycleDisplay.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      speedDisplay.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      beatsMinuteSlider.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      beatsCycleSlider.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      beatsMinuteDisplay.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      beatsCycleDisplay.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
   }
   
   
   
   /**
    * Sets the grid bag layout.
    * ESCA-JAVA0076:
    */
   private void gridBagConstraints()
   {
      // Initialize grid bag layout of GUI.
      
      constraints = new GridBagConstraints();
      
      constraints.fill = GridBagConstraints.HORIZONTAL;
      
      
      
      // Grid bag constraints 1.
      
      constraints.insets = new Insets(10,0,0,0);
      
      
      constraints.ipadx = 130; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 2;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 0;
      
      
      contentPane.add(beatDisplay, constraints);
      
      
      
      // Grid bag constraints 2.
      
      constraints.insets = new Insets(10,0,0,0);
      
      
      constraints.ipadx = 130; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 2;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 2; 
      
      constraints.gridy = 0;
      
      
      contentPane.add(cycleDisplay, constraints);
      
      
      
      // Grid bag constraints 3.
      
      constraints.insets = new Insets(0,0,0,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 10;

      constraints.gridwidth = 2;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 1;
      
      
      contentPane.add(beatsMinuteDisplay, constraints);
      

      
      // Grid bag constraints 4.
      
      constraints.insets = new Insets(0,0,0,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 10;

      constraints.gridwidth = 2;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 2; 
      
      constraints.gridy = 1;
      
      
      contentPane.add(beatsCycleDisplay, constraints);
      
      
      
      // Grid bag constraints 5.
      
      constraints.insets = new Insets(0,0,20,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 20;

      constraints.gridwidth = 4;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 2;
      
      
      contentPane.add(speedDisplay, constraints);
      

      
      // Grid bag constraints 6.
      
      constraints.insets = new Insets(0,0,20,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 4;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 3;
      
      
      contentPane.add(beatsMinuteSlider, constraints);
      
      
      
      // Grid bag constraints 7.
      
      constraints.insets = new Insets(0,0,0,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 4;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 4;
      
      
      contentPane.add(beatsCycleSlider, constraints);
      
     
      
      // Grid bag constraints 8.
      
      constraints.anchor = GridBagConstraints.PAGE_END;
      
      constraints.insets = new Insets(0,10,10,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 4;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 1; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 5;
      
      
      contentPane.add(buttonPanel, constraints);   
   }
   
   
   //*************************************** END INITIALISATION CODE ***************************************//
   
   
   //************************************* START INTERFACE EVENT CODE **************************************//
   
   
   private class BeatsMinuteChangeListener implements ChangeListener 
   {
  
     /**
       * This is the slider listener.
       */
      public void stateChanged(ChangeEvent changeEvent) 
      {
        
         Object source = changeEvent.getSource();

         if (source instanceof JSlider) 
         {
            JSlider theJSlider = (JSlider) source;
      
            if (theJSlider.getValueIsAdjusting()) 
            {
               metronome.setBeatsPerMinute(theJSlider.getValue());
               
               speedDisplay.setText(metronome.getSpeedMode());
               
               beatsMinuteDisplay.setText(Integer.toString(metronome.getBeatsPerMinute()));
            }
            
            if (!theJSlider.getValueIsAdjusting()) 
            {
               stopMetronomeTimer(); startMetronomeTimer(); 
            }
         }     
      }
   }
   
   
   
   private class BeatsCycleChangeListener implements ChangeListener 
   {
  
     /**
       * This is the slider listener.
       */
      public void stateChanged(ChangeEvent changeEvent) 
      {
        
         Object source = changeEvent.getSource();

         if (source instanceof JSlider) 
         {
            JSlider theJSlider = (JSlider) source;
      
            if (theJSlider.getValueIsAdjusting()) 
            {
               metronome.setBeatsPerCycle(theJSlider.getValue());
               
               beatsCycleDisplay.setText(Integer.toString(metronome.getBeatsPerCycle()));
            }
         }     
      }
   }
   
   
   
   private class StartButtonListener
   implements ActionListener
   { 
      public void actionPerformed(ActionEvent event)
      {
         startMetronomeTimer();
      }
   }
   
   
   
   private class StopButtonListener
   implements ActionListener
   { 
      public void actionPerformed(ActionEvent event)
      {
         stopMetronomeTimer();
      }
   }
   

   //************************************** END INTERFACE EVENT CODE ***************************************//
   
   
   //**************************************** START PUBLIC METHODS *****************************************//
   
   
   /**
    * Updates the speed display.
    * @param value
    */
   public static void updateBeatDisplay(String value)
   {
      beatDisplay.setText(value);
   }
   
   
   
   /**
    * Updates the cycle display.
    * @param value
    */
   public static void updateCycleDisplay(String value)
   {
      cycleDisplay.setText(value);
   }
   
   
   
   /**
    * Sets the visibility of the frame.
    */
   public void setVisibility()
   {
   
      if (isVisible) {frame.setVisible(true);}
      
      else {frame.setVisible(false);} 
   }
   
   
   
   /**
    * Displays or Hides contents of GUI.
    * @param shown
    */
   public void setContentDisplay(boolean shown)
   {
      contentPane.setVisible(shown);
   }
   
   
   //***************************************** END PUBLIC METHODS ******************************************//
   
   
   //**************************************** START PRIVATE METHODS ****************************************//
   
   
   /**
    * Starts the metronome timer.
    */
   private void startMetronomeTimer()
   { 
      if (!DigitalMetronome.isTimerRunning())
      {
         metronome.setMetronomeTimer
         (
            metronome.calculateTimerSpeed()
         ); 
      }   
   }
   
   
   
   /**
    * Stops the metronome timer.
    */
   private void stopMetronomeTimer()
   {
      if (DigitalMetronome.isTimerRunning())
      {
         metronome.stopTimer();
      }   
   }
   
   
   
   /**
    * Window resize event.
    */
   private void includeResizeEvent()
   {
      this.addComponentListener 
      (
         new ComponentAdapter() 
         {
            public void componentResized(ComponentEvent e) 
            {
               appletDisplayHandler();
            }
          
         }
      ); 
   }
   
   
   
   /**
    * Controls the JApplet component visibility. 
    * ESCA-JAVA0266:
    */
   private void appletDisplayHandler()
   {
      final String ZOOM_WARNING = 
   
      "It appears that web pages in your browser are zoomed out. \n\n" +
      "In order to display the applet, your browser needs to be set \n" +
      "at its default zoom level of 100% or greater. \n\n" +
      "Sorry about that.";
   
   
      if ((getWidth() < FRAME_SIZE_X) || (getHeight() < FRAME_SIZE_Y))
      {
         contentPane.setVisible(false);  
      
         
         if (!zoomWarningShown) 
         {
            Message.showMessage(ZOOM_WARNING, "Browser zoom setting", "warning");
            
            zoomWarningShown = true;
         } 
      }
       
      else 
      {
         contentPane.setVisible(true);
      } 
   }
   
   
   //***************************************** END PRIVATE METHODS *****************************************//
   
   
   //********************************************** START MAIN *********************************************//
   
   
   /**
    * Main
    * @param a
    */
   public static void main(String[] a)
   {
      isApplication = true;
      
      isVisible = true;
      
      new DigitalMetronomeGui();
   }
   
   
   //*********************************************** END MAIN **********************************************//

   
   /**
    * ESCA-JAVA0007:
    */
   public JFrame frame;
   
   private JPanel buttonPanel;
   private JButton start, stop;
   private Container contentPane;
   private boolean zoomWarningShown;
   private DigitalMetronome metronome;
   private GridBagConstraints constraints;
   private static JTextField beatDisplay, cycleDisplay;
   private JSlider beatsMinuteSlider, beatsCycleSlider;
   private JTextField speedDisplay, beatsMinuteDisplay, beatsCycleDisplay;
}


//********************************************** END CLASS **********************************************//