/*
   Copyright (c) 2010 - 2013 Javaika.com by Alexander Wait.
   
   All Rights Reserved.
*/



package required;

import java.io.IOException;
import java.net.URL;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;


//********************************************* START CLASS *********************************************//


/**
 * This class reads and plays audio files.
 * 
 * @author Alexander Wait
 * @version 2010-07-18
 */
public class AudioPlayer 
{
   private static final int BUFFER_SIZE = 128000;
   
   
   /**
    * ESCA-JAVA0057:
    */
   public AudioPlayer() {}
   
   
   //**************************************** START PUBLIC METHODS *****************************************//
    
    
   /**
    * Loads the audio file.
    * ESCA-JAVA0166:
    * ESCA-JAVA0266:
    * @param filename
    */
   public void loadFile(String filename)
   {   
      try 
      {  
         defaultSound = getClass().getResource(filename); 
      
         info = new DataLine.Info(SourceDataLine.class, audioFormat);

         sourceLine = (SourceDataLine) AudioSystem.getLine(info); 
      } 
        
      catch (Exception e) 
      {System.out.println(Tools.thisPathAndLine() + e + "\n");}
      
   }
   
   
   
   /**
    * Reloads the sound file.
    * @throws LineUnavailableException
    * @throws UnsupportedAudioFileException
    * @throws IOException
    */
   public void reload() throws LineUnavailableException, UnsupportedAudioFileException, IOException
   {
      audioStream = AudioSystem.getAudioInputStream(defaultSound);
       
      audioFormat = audioStream.getFormat();
        
      sourceLine.open(audioFormat);
   
      sourceLine.start();   
   }
   
  
   
   /**
    * Plays the audio file.
    * ESCA-JAVA0166:
    * ESCA-JAVA0266:
    */
   public void play() {readBytes();}
   
   
   //***************************************** END PUBLIC METHODS ******************************************//
   
   
   //**************************************** START PRIVATE METHODS ****************************************//
   
   
   /**
    * Reads the bytes from the audio file.
    * ESCA-JAVA0166:
    * ESCA-JAVA0266:
    */
   private void readBytes()
   {
      int nBytesRead = 0;
       
      byte[] abData = new byte[BUFFER_SIZE];
         
        
      try 
      {
         while (nBytesRead != -1) 
         {
            nBytesRead = audioStream.read(abData, 0, abData.length);
            
            
            if (nBytesRead >= 0) 
            {
               sourceLine.write(abData, 0, nBytesRead);
            } 
         }
         
         if (nBytesRead == -1) {sourceLine.drain(); sourceLine.close();}      
      }
    
      catch (Exception e) 
      {System.out.println(Tools.thisPathAndLine() + e + "\n");} 
   
   }
   
   
   //***************************************** END PRIVATE METHODS *****************************************//
    
   
   private URL defaultSound;
   private DataLine.Info info;
   private AudioFormat audioFormat;
   private SourceDataLine sourceLine;
   private AudioInputStream audioStream;
}


//********************************************** END CLASS **********************************************//