/*
   Copyright (c) 2010 - 2013 Javaika.com by Alexander Wait.
   
   All Rights Reserved.
*/



package required;

import java.util.Timer;
import java.util.TimerTask;

import main.DigitalMetronomeGui;


//********************************************* START CLASS *********************************************//


/**
 *  This class creates a basic Metronome.
 *
 *  @author Alexander Wait
 *  @version 2012-05-05
 */
public class DigitalMetronome
{
   private static final int MAX_BEATS_PER_MINUTE = 190;
   
   private static final int DEFAULT_BEATS_FACTOR = 50;
   
   private static final int DEFAULT_BEATS_PER_CYCLE = 50;
   
   
   //************************************** START INITIALISATION CODE **************************************//
   

   /**
    * Application Constructor.
    */
   public DigitalMetronome()
   {
      player = new AudioPlayer();
      
      
      setBeatsPerCycle(DEFAULT_BEATS_PER_CYCLE);
      
      setBeatsPerMinute(DEFAULT_BEATS_FACTOR);
   }
   
   
   
   //*************************************** END INITIALISATION CODE ***************************************//
   
   
   //**************************************** START PUBLIC METHODS *****************************************//
   

   /**
    * Returns the speed mode.
    * ESCA-JAVA0076:
    * @return
    */
   public String getSpeedMode()
   {
      String mode = null;
      
      if ((getBeatsPerMinute() >= 1) && (getBeatsPerMinute() <= 20))
      {
         mode = "Larghissimo";
      }
      
      else if ((getBeatsPerMinute() >= 21) && (getBeatsPerMinute() <= 40))
      {
         mode = "Largamente";
      }
      
      else if ((getBeatsPerMinute() >= 41) && (getBeatsPerMinute() <= 60))
      {
         mode = "Largo";
      }
      
      else if ((getBeatsPerMinute() >= 61) && (getBeatsPerMinute() <= 66))
      {
         mode = "Larghetto";
      }
      
      else if ((getBeatsPerMinute() >= 67) && (getBeatsPerMinute() <= 76))
      {
         mode = "Adagio";
      }
      
      else if ((getBeatsPerMinute() >= 77) && (getBeatsPerMinute() <= 80))
      {
         mode = "Adagietto";
      }
      
      else if ((getBeatsPerMinute() >= 81) && (getBeatsPerMinute() <= 108))
      {
         mode = "Andante";
      }
      
      else if ((getBeatsPerMinute() >= 109) && (getBeatsPerMinute() <= 120))
      {
         mode = "Moderato";
      }
      
      else if ((getBeatsPerMinute() >= 121) && (getBeatsPerMinute() <= 168))
      {
         mode = "Allegro";
      } 
      
      else if ((getBeatsPerMinute() >= 169) && (getBeatsPerMinute() <= 190))
      {
         mode = "Presto";
      }

      return mode;
   
   }
   
   
   
   /**
    * Sets the allowed beats per minute.
    * @param value
    */
   public void setBeatsPerMinute(int value) 
   {
      double corrected = (value / 100.00) * MAX_BEATS_PER_MINUTE;
      
      beatsPerMinute = (int)corrected;
   }
   
   
   
   /**
    * Cancels the timer.
    */
   public void stopTimer()
   {
      cycle = 0; beat = 0;
   
      timer.cancel(); timerRunning = false;
      
      
      DigitalMetronomeGui.updateBeatDisplay(Integer.toString(beat));
      
      DigitalMetronomeGui.updateCycleDisplay(Integer.toString(cycle));
   }
   
   
   
   /**
    * Initializes the metronome timer.
    * @param speed
    */
   public void setMetronomeTimer(int speed)
   {    
      timer = new Timer(); 
      
      timerRunning = true;
      
      player.loadFile("/files/audio/MetronomeTicker.wav");
      
      timer.schedule(new RunPlayer(), 500, speed);
      
   }
   
   
   
   /**
    * Calculates the timer speed.
    * @return
    */
   public int calculateTimerSpeed()
   {   
      double ratio = (double)DateAndTime.getSecondsPerMinute() 
      
      / (double)getBeatsPerMinute();
    
      
      return (int)(ratio * 1000);
   }
   
   
   
   /**
    * Tests if timer already running.
    * @return
    */
   public static boolean isTimerRunning()
   {
      return timerRunning;
   }
   
   
   
   /**
    * Sets the beats per cycle.
    * @param beats
    */
   public void setBeatsPerCycle(int beats) {beatsPerCycle = beats;}
   
   
   
   /**
    * Get the beats per cycle.
    * @return
    */
   public int getBeatsPerCycle() {return beatsPerCycle;}
   
   
   
   /**
    * Get the beats per minute.
    * @return
    */
   public int getBeatsPerMinute() {return beatsPerMinute;}
   
   
   
   /**
    * Get the beat.
    * @return
    */
   public int getBeat() {return beat;}
   
   
   
   /**
    * Get the cycle.
    * @return
    */
   public int getCycle() {return cycle;}
   
   
   
   //***************************************** END PUBLIC METHODS ******************************************//


   //**************************************** START PRIVATE METHODS ****************************************//
   
   
   private class RunPlayer extends TimerTask
   {

      /**
       * Runs the metronome player.
       * ESCA-JAVA0266:
       * ESCA-JAVA0166:
       */
      public void run()
      {
         try {player.reload();}
         
         
         catch (Exception e)
         {System.out.println(Tools.thisPathAndLine() + e);}
         
        
         if (getBeat() > getBeatsPerCycle()) {cycle ++; beat = 1;}
         
         
         DigitalMetronomeGui.updateBeatDisplay(Integer.toString(beat));
         
         DigitalMetronomeGui.updateCycleDisplay(Integer.toString(cycle));
         
         
         player.play(); 
         
         beat ++;
      }
   }
   
   
   //***************************************** END PRIVATE METHODS *****************************************//
   
   
   private Timer timer;
   private AudioPlayer player;
   private static boolean timerRunning;
   private int beat, cycle, beatsPerCycle, beatsPerMinute;
}


//********************************************** END CLASS **********************************************//